  ' ************************************************
  ' **  Name:          RSA2000.BAS                **
  ' **  Type:          Main program               **
  ' **  Module:        RSA2000                    **
  ' **  Language:      Microsoft QuickBASIC 4.00  **
  ' **                                            **
  ' **    Y2k Year 2000 see below - search Y2K    **
  ' **    For Use With ADAC 5500HR A/D Card       **
  ' ************************************************
  ' USAGE:           Main Module with no command line parameters
  '
  ' REQUIREMENTS:    MIXED_AD.QLB/.LIB
  '                  ADAC.DAT                ADAC Card setup file
  '                  Mouse (optional)
  ' FILES:           WINDOWS.BAS
  '                  BITS.BAS
  '                  BIOSCALL.BAS
  '                  MOUSSUBS.BAS
  '                  KEYS.BAS
  ' PARAMETERS:      (none)
  ' VARIABLES:       w1         Structure of type WindowsType
  '                  w2         Structure of type WindowsType
  '                  w3         Structure of type WindowsType
  '                  w1Text$()  Strings to display in first window
  '                  w2Text$()  Strings to display in second window
  '                  w3Text$()  Strings to display in third window
  '                  w1Title$   Title string for first window
  '                  w1Prompt$  Prompt string for first window
  '                  w2Title$   Title string for second window
  '                  w2Prompt$  Prompt string for second window
  '                  w3Title$   Title string for third window
  '                  arrow$     String showing up and down arrows
  '                  entSymbol$ String showing the Enter key symbol
  '                  w3Prompt$  Prompt string for third window
  '                  i%         Looping index
  '                  t0         Timer value
  
  
  ' Define color constants
    CONST BLACK = 0
    CONST BLUE = 1
    CONST GREEN = 2
    CONST CYAN = 3
    CONST RED = 4
    CONST MAGENTA = 5
    CONST BROWN = 6
    CONST WHITE = 7
    CONST BRIGHT = 8
    CONST BLINK = 16
    CONST YELLOW = BROWN + BRIGHT
 
  ' Define logical constants
    CONST FALSE = 0
    CONST TRUE = NOT FALSE

    TYPE WindowsType
        action       AS INTEGER
        edgeLine     AS INTEGER
        row          AS INTEGER
        col          AS INTEGER
        fgdEdge      AS INTEGER
        bgdEdge      AS INTEGER
        fgdBody      AS INTEGER
        bgdBody      AS INTEGER
        fgdHighlight AS INTEGER
        bgdHighlight AS INTEGER
        fgdTitle     AS INTEGER
        bgdTitle     AS INTEGER
        fgdPrompt    AS INTEGER
        bgdPrompt    AS INTEGER
        ReturnCode   AS INTEGER
    END TYPE

  
  ' Functions
    DECLARE FUNCTION InKeyCode% ()
  
  ' Subprograms
    DECLARE SUB Windows (W AS WindowsType, wText$(), wTitle$, wPrompt$)
    DECLARE SUB VGAplot (W AS WindowsType, wText$(), wTitle$, wPrompt$)
    DECLARE SUB windowspop ()
    DECLARE SUB VideoState (Mode%, columns%, page%)
    DECLARE SUB Mouse (m1%, m2%, m3%, m4%)
    DECLARE SUB MouseMickey (horizontal%, vertical%)
    DECLARE SUB MouseNow (leftButton%, rightButton%, xMouse%, yMouse%)
    DECLARE SUB PrintScreen ()
   
    DEFINT I-M          ' Use some Fortran conventions

'   ADLIB function declarations for Microsoft QuickBASIC
'   For ADAC 5500HR Data Acquisition Board

    DECLARE FUNCTION AIN% (BYVAL BuffSEG%, BYVAL BuffOFFS%, BYVAL Count%, BYVAL Chan%, BYVAL Trig%, BYVAL Gain%, BYVAL Unit%)
    DECLARE FUNCTION AINI% (BYVAL BuffSEG%, BYVAL BuffOFFS%, BYVAL Count%, BYVAL Chan%, BYVAL Trig%, BYVAL Gain%, BYVAL Device%, SEG Flag%)
    DECLARE FUNCTION AINR% (BYVAL BuffSEG%, BYVAL BuffOFFS%, BYVAL Count%, BYVAL Chan%, BYVAL Trig%, BYVAL Gain%, BYVAL Device%, SEG Flag%)
    DECLARE FUNCTION AOT% (BYVAL BuffSEG%, BYVAL BuffOFFS%, BYVAL Count%, BYVAL Chan%, BYVAL Device%)
    DECLARE FUNCTION DIN% (BYVAL BuffSEG%, BYVAL BuffOFFS%, BYVAL Count%, BYVAL Chan%, BYVAL Device%)
    DECLARE FUNCTION DIND% (BYVAL BuffSEG%, BYVAL BuffOFFS%, BYVAL Count%, BYVAL Chan%, BYVAL Device%, SEG Flag%)
    DECLARE FUNCTION DINI% (BYVAL BuffSEG%, BYVAL BuffOFFS%, BYVAL Count%, BYVAL Chan%, BYVAL Device%, SEG Flag%)
    DECLARE FUNCTION DINR% (BYVAL BuffSEG%, BYVAL BuffOFFS%, BYVAL Count%, BYVAL Chan%, BYVAL Device%, SEG Flag%)
    DECLARE FUNCTION DOT% (BYVAL BuffSEG%, BYVAL BuffOFFS%, BYVAL Count%, BYVAL Chan%, BYVAL Device%)
    DECLARE FUNCTION DOTD% (BYVAL BuffSEG%, BYVAL BuffOFFS%, BYVAL Count%, BYVAL Chan%, BYVAL Device%, SEG Flag%)
    DECLARE FUNCTION DOTI% (BYVAL BuffSEG%, BYVAL BuffOFFS%, BYVAL Count%, BYVAL Chan%, BYVAL Device%, SEG Flag%)
    DECLARE FUNCTION DOTR% (BYVAL BuffSEG%, BYVAL BuffOFFS%, BYVAL Count%, BYVAL Chan%, BYVAL Device%, SEG Flag%)
    DECLARE FUNCTION CIN% (BYVAL BuffSEG%, BYVAL BuffOFFS%, BYVAL Mode%, BYVAL Chan%, BYVAL Device%)
    DECLARE FUNCTION COT% (BYVAL BuffSEG%, BYVAL BuffOFFS%, BYVAL Mode%, BYVAL Chan%, BYVAL Device%)
    DECLARE FUNCTION BCLR% (BYVAL Mask%, BYVAL Chan%, BYVAL Device%)
    DECLARE FUNCTION BSET% (BYVAL Mask%, BYVAL Chan%, BYVAL Device%)
    DECLARE FUNCTION BTST% (BYVAL Mask%, SEG State%, BYVAL Chan%, BYVAL Device%)
    DECLARE FUNCTION IOKIL% (BYVAL Unit%)
    DECLARE FUNCTION AINIT% ALIAS "INIT" (BYVAL Fileseg%, BYVAL Fileoffs%)
    DECLARE FUNCTION ARESTORE% ALIAS "RESTORE" ()
 
  ' Data structures
    DIM w1 AS WindowsType
    DIM w2 AS WindowsType
    DIM w3 AS WindowsType
    DIM w4 AS WindowsType
    DIM w5 AS WindowsType
  ' Arrays
    DIM w1Text$(1 TO 5)
    DIM w2Text$(1 TO 17)
    DIM w3Text$(1 TO 11)
    DIM w4Text$(1 TO 20)
    DIM w5Text$(1 TO 5)

  ' Define first window  (ERROR)
    w1.action = 0
    w1.edgeLine = 1
    w1.row = 2
    w1.col = 40
    w1.fgdEdge = YELLOW
    w1.bgdEdge = BLUE
    w1.fgdBody = BRIGHT + WHITE
    w1.bgdBody = BLUE
    w1.fgdHighlight = 0
    w1.bgdHighlight = 0
    w1.fgdTitle = YELLOW
    w1.bgdTitle = BLUE
    w1.fgdPrompt = YELLOW
    w1.bgdPrompt = BLUE
    w1Title$ = " ERROR "
    w1Text$(1) = "Incorrect information entered."
    w1Text$(2) = "Please re-enter."
    w1Prompt$ = ""
  
  ' Define second window   (Phi Window)
    w2.action = 1
    w2.edgeLine = 2
    w2.row = 7
    w2.col = 8
    w2.fgdEdge = CYAN + BRIGHT
    w2.bgdEdge = BLACK
    w2.fgdBody = YELLOW
    w2.bgdBody = BLACK
    w2.fgdHighlight = 0
    w2.bgdHighlight = 0
    w2.fgdTitle = CYAN + BRIGHT
    w2.bgdTitle = BLUE
    w2.fgdPrompt = CYAN + BRIGHT
    w2.bgdPrompt = BLUE
    w2Title$ = " Phi Percentages "
    w2Text$(1) = "This window waits for the user to press"
    w2Text$(2) = "any key before continuing.  The key code"
    w2Text$(3) = "is passed back to the calling program."
    w2Prompt$ = " Press any key to continue. "
  
  ' Define third window  (Selections)
    w3.action = 2
    w3.edgeLine = 2
    w3.row = 7
    w3.col = 15
    w3.fgdEdge = YELLOW
    w3.bgdEdge = WHITE
    w3.fgdBody = BLACK
    w3.bgdBody = WHITE
    w3.fgdHighlight = WHITE + BRIGHT
    w3.bgdHighlight = BLACK
    w3.fgdTitle = YELLOW
    w3.bgdTitle = WHITE
    w3.fgdPrompt = YELLOW
    w3.bgdPrompt = WHITE
    w3Title$ = " Select the action you wish to take. "
    arrows$ = CHR$(24) + " " + CHR$(25) + " "
    entSymbol$ = CHR$(17) + CHR$(196) + CHR$(217)
    w3Prompt$ = " <Character> " + arrows$ + entSymbol$ + " or use mouse "
    w3Text$(1) = "1. REPLOT - Oops, I need to look again. "
    w3Text$(2) = "2. PRINT & SAVE. Everything looks good."
    w3Text$(3) = "3. PRINT AGAIN. I need another copy."
    w3Text$(4) = "4. PRINT SCREEN. High resolution printout."
    w3Text$(5) = "5. OVERPLOT. Plot prior data over these (3 Max)."
    w3Text$(6) = "6. PLOT MAXIMUM SCALE - fully expanded"
    w3Text$(7) = "7. PLOT MINIMUM SCALE - Out to 4 Phi +"
    w3Text$(8) = "8. RESTART - Throw out data. Sample no good"
    w3Text$(9) = "9. NEXT SAMPLE. All done, ready for next sample."
    w3Text$(10) = "A. FILTER - Filter noise and recalculate Phi%."
    w3Text$(11) = "B. QUIT - Exit to system."

  ' Define fourth window   (Plot Screen)
    w4.action = 3
    w4.edgeLine = 2
    w4.row = 1
    w4.col = 1
    w4Title$ = " This is the plot window "
    w4Text$(1) = ""
    w4Text$(2) = ""
    w4Text$(3) = ""
    w4Text$(4) = ""
    w4Text$(5) = ""
    w4Text$(6) = ""
    w4Text$(7) = ""
    w4Text$(8) = ""
    w4Text$(9) = ""
    w4Text$(10) = ""
    w4Text$(11) = ""
    w4Text$(12) = ""
    w4Text$(13) = ""
    w4Text$(14) = ""
    w4Text$(15) = ""
    w4Text$(16) = ""
    w4Text$(17) = ""
    w4Text$(18) = ""
    w4Text$(19) = ""
    w4Text$(20) = ""
    LRarrows$ = CHR$(27) + " " + CHR$(26) + " "
 
  ' Define fifth window (information)
    w5.action = 0
    w5.edgeLine = 1
    w5.row = 2
    w5.col = 40
    w5.fgdEdge = YELLOW
    w5.bgdEdge = BLUE
    w5.fgdBody = BRIGHT + WHITE
    w5.bgdBody = BLUE
    w5.fgdHighlight = 0
    w5.bgdHighlight = 0
    w5.fgdTitle = YELLOW
    w5.bgdTitle = BLUE
    w5.fgdPrompt = YELLOW
    w5.bgdPrompt = BLUE
    w5Title$ = " INFORMATION "
    w5Text$(1) = "Dummy Info."
    w5Text$(2) = "Dummy Info."
    w5Prompt$ = ""
 
    
 
  

'       *********************************************************
'       *                                                       *
'       *       RSA2000  QuickBasic Version       03/31/2000    *
'       *       A. Eliason    Eliason Data Services             *
'       *                 (508) 477-1400                        *
'       *********************************************************
'
'   Mods for version RSAQB93
'   Changed Lab No. input so alpha portion is always upper case
'   Changed tolerance in Phi % sunnation so 99.9% = 100%
'   Changed write routine for RSA.NDX file to include "RSA" in position
'   used by CLTR.NDX for tube size.
'   Fixed ESC option during sample introduction wait period

'   Y2K Program checked and annotated for Year 2000 problems 07-27-98 AHE
'   Y2K modifications made 10-10-98 AHE
'   To find, search Y2K in this code
'
'   3/2000 Program modified for use with ADAC 5500 HR Data Acquisiton Board
'   all old DAC code removed.
'   IMPORTANT These routines & hardware require an interrupt level 3-7
'   IRQ 4 uas used for the tests. The IRQ must be reserved by the host
'   computer OS exclusively for this card.
    
     Version$ = "03/31/2000 ADAC Y2k"
     NoADC = FALSE   'True for tests with no A/D interface
     ICNT = 4800                    'sample array size
     Iplot = 7                      'PLOT SCALE FACTOR 1-10
     TFAC = .05                     '20 samples/second
     FFAC = 10                      'POPPE TIME DELAY FACTOR
     WIDTH LPRINT 255               'For Epson Mx/Fx Series printer
     MINC = INT(50 / TFAC + 15)     '1015-3615
     MAXC = INT(180 / TFAC + 15)    'MIN AND MAX SAMPLE COUNT
     ITENTH = MAXC / 10
     ln1$ = "PLEASE "
     ln2$ = "ENTER"
    
     '
     DIM time(13), Mo$(12), IV(4850), IVP(10), GF(5), PHIT(14), PHIV(12)
     DIM PHIP(12), PHIR(6), QP$(9), IVOP(4850)
     DIM Mo(12), D$(20)
     
     DATA "January","February","March","April"
     DATA "May","June","July","August","September","October","November"
     DATA "December"
phidata:
     DATA 4.2,5.1,6.0,7.1,9,11.3,14.5,21,31,50,94,160,2000
     
     FOR i = 1 TO 12
          READ Mo$(i)
     NEXT i
     FOR i = 1 TO 13
          READ x
          PHIT(i) = x + .5    'add .5sec for precount data. This is NOT poppe factor.
     NEXT i

'  Get the date & time
     GOSUB clock
     PLT = -1: REM PLOT


'********************** Setup A/D Converter Card *********************
    CLS
    IF NoADC THEN GOTO Intro
    PRINT : PRINT : PRINT "RSA Initiaization - RSA2000:  "; Version$
    PRINT "Testing Data Acquisition Device"
'
'   This test routine performs Analog to Digital
'   conversions on Logical Unit 1, converts the raw binary
'   information to engineering units and displays the
'   information on the screen.
'
'   If Ctrl-Break is pressed, the program will terminate
'
'   NOTE: This program will generate an error unless a hardware
'         configuration file named ADAC.DAT has been created.  The
'         hardware described by ADAC.DAT must also be present.
'
'   Release some memory for use by ADLIB (non-BASIC routines). At least 2K
'   should be freed.

    TEMP! = SETMEM(-8000)
    Fosc% = &HC350              ' 50,000 time units per sample at 1 MHz = 20 per sec
                                ' C350h=50000d unsigned integer > 32767
    Mode = 0                    ' Constant Frequency
    IChan = 0                   ' Just first chan (0)
    ILun = 1                    ' Logical Unit # 1
    
    GOSUB InitA2D
    PRINT "AINIT Done."
    PRINT "COT Setup Done."
    SLEEP 1
'   Scan analog input channels 100 times

    DIM Buff%(8)
    ICount = 8                          ' 8 each time
    IChan = -7                          ' 0 to 7
    ITrig = 0                           ' Fast as card can convert
    IGain = 0                           ' Must be 0 fo this card
    ILun = 1                            ' Logical Unit 1
    Factor! = 20! / 65536!              ' for 16 bit +-10 Volt

    FOR i = 1 TO 100
        GOSUB ReadA2D
        ' Display acquired data
        FOR j = 0 TO 7
            VOLTAGE! = Buff%(j) * Factor!
            LOCATE 7 + j, 5, 0
            PRINT USING "Voltage(##) = ###.###"; j; VOLTAGE!
        NEXT j
        FOR j = 1 TO 32000      'waste some time
        NEXT j
    NEXT i
    SLEEP 1                     'not this one
   
    PRINT "Interrupt test routines starting:": PRINT

    FOR i = 0 TO 4850               ' Set the raw data buffer
         IV(i) = -32768             ' to minimum
         IVOP(i) = 0                ' and the overplot buffer to 0
    NEXT i

    ICount = 100               ' do 20/sec for 5 seconds
    IChan = 0
    ITrig = 1
    Iflag = 0
    IVOffset = 11
    GOSUB ScanA2D

    PRINT "AINI Initiated. This test will take exactly 5 seconds. If you wish to"
    PRINT "skip/terminate the test, press any key."
    PRINT
    DO WHILE Iflag = 0
         FOR i = 10 TO 111
              j! = 0
              DO WHILE IV(i) = -32768 AND j! < 200000
              j! = j! + 1
              LOOP
              LOCATE 20, 5, 0
              PRINT USING "#### ###### ###.####"; i; IV(i); IV(i) * Factor!;
              a$ = INKEY$
              IF a$ <> "" THEN EXIT DO
         NEXT i
         IF IV(11) = -32768 OR IV(110) = -32768 THEN
            PRINT " AINI Scan failed in EDS test routine."
            END
         ELSE
            PRINT " AINI Scan Succeeded."
         END IF
    LOOP
    
    GOSUB StopA2D
    PRINT "ARESTORE Done. Tests completed."
   
    INPUT "Press 'Enter'to start main program.", a$
    IF a$ = "AHE" THEN
        FOR i = 0 TO 120
             PRINT i; " "; IV(i); "    ";
        NEXT i
        PRINT
        'INPUT "Want Test mode?"; a$
        'IF LEFT$(a$, 1) = "Y" OR LEFT$(a$, 1) = "y" THEN NoADC = TRUE
        SLEEP 10
    END IF
'   Set up for the real data
    ICount = 4800               ' 20/sec x 60 sec x 4 min = maximum sample count
    ITrig = 1                   ' Trig mode 1a
    Iflag = 0                   ' Iflag is incremented when interrupt a/d done
    Fosc% = &HC350              ' 50,000 time units per sample at 1 MHz = 20 per sec
                                ' C350h=50000d unsigned integer > 32767
    Mode = 0                    ' Constant Frequency
    IChan = 0                   ' Just first chan (0)
    IGain = 0                   ' Must be 0 fo this card
    ILun = 1                    ' Logical Unit 1
    Factor! = 20! / 65536!      ' for 16 bit +-10 Volt


Intro:
     '************** INTRODUCTION **************
     ' Initialize the display
     SCREEN 0, , 0, 0
     WIDTH 80
     CLS
     FOR i% = 1 TO 15
         PRINT STRING$(80, 178)
     NEXT i%

     LOCATE 6, 10
     PRINT " RSA Data Logging Program   Version RSA2000  "; Version$
     LOCATE 7, 15
     PRINT " Eliason Data Services     (508) 477-1400     04/13/94 "
     LOCATE 9, 25
     PRINT " "; t$; "   "; Mo$(Mo); " "; DA$; ", "; y$; " "
     LOCATE 17, 1
     PRINT "Please answer query questions with 'Y' for Yes, 'N' for No, or the information  requested. You may also just hit the 'Enter' key to accept the default reply    which is contained within square brackets, i.e. [ BLA ]."
     PRINT : PRINT "Please set printer to proper 'Top Of Form' then turn it ON."
     INPUT "When done, please hit 'Enter ' ", a$
     LPRINT CHR$(27); "C"; CHR$(0); CHR$(11); : REM SET EPSON FOR 11 IN. FORM LENGTH
     CLS
    
Begin:
    IF NOT NoADC THEN GOSUB InitA2D
    FOR i = 0 TO 4850               ' Set the raw data buffer
         IV(i) = -32768             ' to minimum
         'IVOP(i) = 0                ' and the overplot buffer to 0
    NEXT i

'************* GET THE HEADER DATA **************
     VER = 0: sid$ = "PLEASE ENTER": IGF = -1
headerinfo:
     GOSUB clock
     PRINT t$; "   "; D$
     PRINT
     PRINT "Enter 'Q' to QUIT"
     PRINT "Operator Name: ["; OPN$; : INPUT "] ", a$
     IF a$ <> "" THEN
          IF a$ = "Q" OR a$ = "q" THEN SYSTEM          'quit to system
          OPN$ = a$
          PRINT "  Hello, "; OPN$; ". Please run one or two test samples before you start"
          PRINT "  the real ones."
     END IF
     PRINT
     PRINT "Requested by:  ["; SN$;
     INPUT "] ", a$
     IF a$ <> "" THEN SN$ = a$
     PRINT
     PRINT "Cruise I.D.:   ["; CID$;
     INPUT "] ", a$
     IF a$ <> "" THEN CID$ = a$
     PRINT
     PRINT "Project I.D.:  ["; PID$;
     INPUT "] ", a$
     IF a$ <> "" THEN PID$ = a$
sid:
     PRINT : PRINT "Sample I.D.:   ["; sid$; : INPUT "] ", a$
     IF a$ = "" AND VER = 0 THEN
          GOSUB ErrWindow1
          GOTO sid
     END IF
     IF LEN(a$) > 16 OR LEN(sid$) > 16 THEN
          GOSUB ErrWindow1
          PRINT "That entry is too long! (16 char max.)"
          GOTO sid
     END IF
     IF a$ = "" AND VER > 0 THEN GOTO labno
     sid$ = a$
   
labno:
     PRINT
     PRINT "Lab Number: (format 'AAnnn') ["; ln1$; ln2$;
     INPUT "] ", a$
     lnerr% = 0
     IF a$ = "" THEN GOTO lablen
     IF LEN(a$) < 3 OR LEN(a$) > 5 THEN lnerr% = 1
     ln1$ = UCASE$(LEFT$(a$, 2))                  ' Upper case AHE 8-23-93
     ln2$ = MID$(a$, 3, 3)
lablen:
     IF (LEFT$(ln1$, 1) < "A" OR LEFT$(ln1$, 1) > "Z") THEN lnerr% = 1
     IF (RIGHT$(ln1$, 1) < "A" OR RIGHT$(ln1$, 1) > "Z") THEN lnerr% = 1
     IF VAL(ln2$) < 1 OR VAL(ln2$) > 999 THEN lnerr% = 1
     IF lnerr% THEN
          GOSUB ErrWindow1
          GOTO labno
     END IF
     IF LEN(ln1$) <> 2 OR LEN(ln2$) > 3 OR VAL(ln2$) = 0 THEN
          GOSUB ErrWindow1
          GOTO labno
     END IF

tdf:
     PRINT : PRINT "Time delay factor? Positive integer only. 1 unit = .05 seconds ["; FFAC;
     INPUT "] ", a$: IF a$ <> "" THEN FFAC = INT(VAL(a$))
     IF FFAC < 0 THEN
          GOSUB ErrWindow1
          PRINT "Positive Value Only!": GOTO tdf
     END IF
     RESTORE phidata
     FOR i = 1 TO 13
          READ x
          PHIT(i) = x + FFAC * TFAC
     NEXT i                                  ' ADD PRECOUNT TIME
weights:
     PRINT
     PRINT "Sample Weight (Grams)? ["; SW;
     INPUT "] ", a$
     IF a$ <> "" THEN SW = VAL(a$)
     PRINT "Coarse weight (Grams)? ["; CW;
     INPUT "] ", a$
     IF a$ <> "" THEN CW = VAL(a$)
     PRINT "Sand Weight (Grams)? ["; SNDW;
     INPUT "] ", a$
     IF a$ <> "" THEN SNDW = VAL(a$)
     IF CW > SW OR SNDW > CW THEN
          PRINT "Sample Weight must be >= Coarse Weight and"
          PRINT "Coarse Weight must be >= Sand Weight."
          GOSUB ErrWindow1
          GOTO weights
     END IF
    
     PRINT "Is there a gravel fraction? ["; : IF IGF THEN PRINT "Y";
     IF NOT IGF THEN PRINT "N";
     INPUT "] ", a$
     IF a$ = "Y" OR a$ = "y" THEN IGF = -1
     IF a$ = "N" OR a$ = "n" THEN IGF = 0
' ELSE REMAINS THE SAME
     IF NOT IGF THEN GOTO verify
gfp:
     PRINT "Enter Gravel Fraction percentage for each PHI Class."
     TGF = 0
     FOR i = 1 TO 5
          PRINT -1 * i; " PHI % = ["; GF(i); : INPUT "] ", a$
          IF a$ <> "" THEN GF(i) = VAL(a$)
          TGF = TGF + GF(i): REM TOTAL GRAVEL FRACTION
     NEXT i
     IF TGF < 100.11 AND TGF > 99.89 THEN      ' changed to .1% 8-93 AHE
          TGF = 100!
          GOTO verify
     END IF
     GOSUB ErrWindow1
     PRINT "WARNING! Gravel fraction must sum to 100 + or - 0.1"
     PRINT
     GOTO gfp

verify:
     IF VER = 0 THEN
          VER = VER + 1
          CLS
          PRINT "PLEASE VERIFY THE DATA"
          PRINT
          GOTO headerinfo
     END IF
     PRINT : INPUT "All entries correct? [Y] ", a$
     IF a$ = "" OR a$ = "y" OR a$ = "Y" THEN
          VER = 0
     ELSE
          VER = VER + 1
          CLS
          PRINT "PLEASE CORRECT THE BAD ITEM(S)"
          PRINT
          GOTO headerinfo
     END IF

GETSAMP:
  
' ************** GET THE SAMPLES *************************
     Itest = 0  'itest=1 for calculated test input
     IF NoADC THEN Itest = 1
     IVPC = 0

     ot = TIMER
     bt = TIMER
     IGR = 0
     VI = 1.575

     'set up a plot window for raw data
     w4Title$ = " SAMPLE " + ln1$ + ln2$ + " - " + sid$ + " "
     w4Prompt$ = " Use ESC to terminate sampling "
     SCREEN 10, , 0, 0
     VGAplot w4, w4Text$(), w4Title$, w4Prompt$
     PCOPY 1, 0
     SCREEN , , 0, 0
     MIN = 0
     WINDOW (0, 0)-(MAXC, 4000)'estimated a/d high value =4000
     SLEEP 1
     ICount = 1
     ITrig = 0
     GOSUB readAtoD: REM READ A/D CONVERTER
     LOCATE 2, 5, 0
     PRINT "Waiting for sample introduction "; OPN$; " ..."
getnext:
     FOR i = 1 TO 32000                         ' waste some time
     NEXT i
     OLDERIGR = OLDIGR: OLDIGR = IGR
     GOSUB readAtoD                             ' A/D CONVERTER READ
     LOCATE 2, 60, 0
     PRINT IGR
     IVPC = IVPC + 1: IF IVPC > 10 THEN IVPC = 1
     IVP(IVPC) = IGR: REM SAVE THE PRE-COUNT DATA
     IF IGR > OLDIGR + 20 AND IGR > OLDERIGR + 20 THEN GOTO savedata
                    ' TRY TO AVOID GLITCHES
     ' test for ESC key
     code% = InKeyCode%
     IF code% = 27 THEN GOTO gotem ' test for ESC in case you got hung up in
                                   ' sample introduction loop. AHE 8-93

     ' quickbasic
     WHILE TIMER = ot              ' wait until timer increments
     WEND
     ot = TIMER

     GOTO getnext

' *********** SAVE THE DATA AS INTEGER ARRAY ************
savedata:
     FOR i = 1 TO 10    ' ADD THE PRE-COUNT DATA TO THE ARRAY
        IVPC = IVPC + 1: IF IVPC > 10 THEN IVPC = 1
        IV(i) = IVP(IVPC)
     NEXT i
     IF Itest = 0 THEN                  ' Fill array under interrupts
        ICount = 4800
        ITrig = 1
        GOSUB ScanA2D
     END IF
     SLEEP 1
     i = 1                              ' plot from 1st point
     LOCATE 2, 5, 0                     ' Overwrite with blank space
     PRINT SPC(70);
reget:
     IF Itest = 1 THEN GOSUB readAtoD       ' A/D CONVERTER READ fake for test
     IF Itest = 1 THEN IV(i) = IGR          ' Put the fake in IV()
     j! = 0
     DO WHILE IV(i) = -32768 AND j! < 200000    ' Wait til IV() has a new value
     j! = j! + 1                                ' but don't wait forever!
     LOOP
     LOCATE 2, 50, 0
     PRINT USING "#### ###### ##.####"; i; IV(i); IV(i) * Factor!;
          
    'plot
     IF Itest = 1 THEN
          IV(i) = IGR                    ' test
     ELSE
          IGR = IV(i)                    ' real data
     END IF
     Jplot = IGR                        ' New 3-2000
     DO WHILE Jplot > 4000              ' !!!! more work for -readings etc
     Jplot = Jplot - 4000
     LOOP
     PSET (i, Jplot)
   
    ' test for ESC key
     code% = InKeyCode%
     IF code% = 27 THEN
        Count% = i
        GOTO gotem
     END IF
    
     IF i < MINC THEN GOTO timinc       'SKIP END TEST
     IF i > MAXC THEN
        Count% = i
        GOTO gotem                      'TIMEOUT
     END IF
    ' Test value to see if it is at baseline and is flat
     IF IGR < (IV(1) + 1) AND IGR = LASTIGR THEN
        Count% = i
        'PRINT "Value at Baseline IV(1), IGR "; IV(1); IGR
        GOTO gotem
     END IF
timinc:
     WHILE TIMER = ot
     WEND
     ot = TIMER
     i = i + 1: LASTIGR = IGR: GOTO reget

gotem:
     IF NOT Itest THEN GOSUB StopA2D
     PRINT "Count = "; Count%
     PRINT
     ICount = Count%


scale:                                  ' ENTRY POINT FOR REPRINT OF DATA

' **************** SCALE THE DATA ******************
     PRINT "SCALING DATA"
     x = 0
     FOR i = ICount - 9 TO ICount
          x = x + IV(i)
     NEXT i
     BL = x / 10
     PRINT "BASELINE = "; BL
     MAX = 0: MIN = 4000
     FOR i = 1 TO ICount
          IF IV(i) > MAX THEN MAX = IV(i)
          IF IV(i) < BL - 2 THEN PRINT "DATA POINT "; i; " = "; IV(i); " REPLACED WITH "; BL: IV(i) = BL' WIPE OUT NEGATIVE GLITCHES
          IF IV(i) < MIN THEN MIN = IV(i)
     NEXT i
     RANGE = MAX - MIN
     RINC = RANGE / 200: REM Y MAX= 200
     OFFSET = 0 - MIN
     GOSUB PhiCalc: REM COMPUTE PHI
     ifin = 0       'reset ifin (data stored flag)

     '   ********************* SCREEN PLOT ************************
ScreenPlot:
     w4Title$ = " " + ln1$ + ln2$ + " - " + sid$ + " "
     w4Prompt$ = " Use ESC for options or " + LRarrows$ + "to rescale "
     SCREEN 10, , 0, 0
     VGAplot w4, w4Text$(), w4Title$, w4Prompt$
     PCOPY 1, 0
     SCREEN , , 0, 0
     D$(0) = "HU3EFD3GBR5"         '    0
     D$(1) = "NLNRU5GBR6BD4"       '    1
     D$(2) = "NRL2UE2UHGBR6BD4"    '    2
     D$(3) = "LR2U2HEUL2BR6BD5"    '    3
     D$(4) = "BRUDU5BL2D3R2BF2BR1" '    4
     D$(5) = "BLR1EU2L2U2R2BR4BD5" '    5
     D$(6) = "HUEFDGHU3EBR5BD5"    '    6
     D$(7) = "UEU3L2DBR6BD4"       '    7
     D$(8) = "LU5R2D2NLD3BR4"      '    8
     D$(9) = "EU3HGDFBR5BD2"       '    9
     D$(10) = "UDBR5"              '    .
     D$(11) = "BU3LR2BF2BR"        '    -
     D$(12) = "BLE4BL2GD2FEU2HBF4BR2"' phi
     D$(13) = "C0DL4R24UL24UR24UL24UR24UL24UR24UL24UR24D7L24C3"

PlotPhi:
     IPC = 1
     PC = -1.5
     WINDOW (0, MIN)-(ICount, MAX)
     CLS
     GOSUB phiwindow
    
     FOR i = 1 TO ICount
          PSET (i, IV(i))
          IF (i * TFAC) >= PHIT(IPC) THEN
              LINE (i, MIN)-(i, MAX)
              IPC = IPC + 1
          END IF
          NEXT i
     IPC = 1
     FOR i = 1 TO ICount
          IF (i * TFAC) >= PHIT(IPC) THEN
              PSET (i, MIN + 8 * (RANGE / 300) * IPC)
              DRAW D$(13) 'space
              IF PC < 0 THEN DRAW D$(11)  'minus
              DRAW D$(INT(ABS(PC))) + D$(10)
              IF IPC MOD 2 THEN DRAW D$(5) + D$(12)
              IF IPC MOD 2 = 0 THEN DRAW D$(0) + D$(12)

              IPC = IPC + 1
              PC = PC + .5
          END IF
     NEXT i
     PSET (ICount / 2 + 10, MAX / 2)
     OPnum% = 0     'reset overplot #

phidisp:
     VGAplot w2, w2Text$(), w2Title$, w2Prompt$
     code% = w2.ReturnCode
     windowspop
     IF code% = 19200 OR code% = 19712 THEN GOTO arrowtest

keyin:
     DO
     code% = InKeyCode%
     LOOP UNTIL code%
    
     'test enter
     IF code% = 13 THEN GOTO phidisp

arrowtest:
     'test left arrow
     IF code% = 19200 THEN
          ICount = ICount - ITENTH
          IF ICount < 0 THEN BEEP: ICount = ICount + ITENTH
          GOTO PlotPhi
     END IF
    
     'test right arrow
     IF code% = 19712 THEN
          ICount = ICount + ITENTH
          IF ICount > MAXC THEN BEEP: ICount = ICount - ITENTH
          GOTO PlotPhi
     END IF
    
     'test ESC
PullSelect:
     IF code% = 27 THEN
          '  pull down select menu
          VGAplot w3, w3Text$(), w3Title$, w3Prompt$
          windowspop
     ELSE GOTO keyin
     END IF
     
     code% = w3.ReturnCode
     SELECT CASE code%
     CASE -1
          GOTO keyin            'look again
     CASE 1
          GOTO ScreenPlot       'plot again

     CASE 2                     'print & save
          IF ifin = 1 THEN
               w1Text$(1) = "You have already saved data."
               w1Text$(2) = "Please select another option"
               GOSUB ErrWindow2
               code% = 27
               GOTO PullSelect
          END IF
          w5Title$ = " PLOTTING "
          w5Text$(1) = "Plotting to Printer."
          w5Text$(2) = "Please Wait."
          VGAplot w5, w5Text$(), w5Title$, w5Prompt$
          GOSUB PrintHeader
          GOSUB PrintPhi
          GOSUB SaveRawDat
          GOSUB PrintPlot
          windowspop
          GOSUB ToDisk
          ifin = 1
          code% = 27
          GOTO PullSelect
     
     CASE 3
          w5Title$ = " PLOTTING "
          w5Text$(1) = "Re-Plotting to Printer."
          w5Text$(2) = "Please Wait."
          VGAplot w5, w5Text$(), w5Title$, w5Prompt$
          GOSUB PrintHeader
          GOSUB PrintPhi
          GOSUB PrintPlot
          windowspop
          code% = 27
          GOTO PullSelect
    
     CASE 4
          w5Title$ = " PRINT SCREEN "
          w5Text$(1) = "This will take several minutes"
          w5Text$(2) = "Please wait while printing."
          VGAplot w5, w5Text$(), w5Title$, w5Prompt$
          GOSUB timeout
          GOSUB timeout
          windowspop
          PrintScreen                        'print screen
          LPRINT CHR$(12); CHR$(27); "@";    'top of form & restore printer
          code% = 27
          GOTO PullSelect
     
     CASE 5              'overplot
          GOSUB GetRawDat
          FOR i = 1 TO OPcount
          PSET (i, IVOP(i))
          IF (i * TFAC) >= PHIT(IPC) THEN
              LINE (i, MIN)-(i, MAX)
              IPC = IPC + 1
          END IF
          NEXT i
          IPC = 1
          GOTO keyin
          
     CASE 6
          ICount = ITENTH
          GOTO PlotPhi
   
     CASE 7
          ICount = MAXC
          GOTO PlotPhi
     
     CASE 8
          GOTO StartOver

     CASE 9
          IF ifin = 0 THEN
               w5Title$ = " PLOTTING "
               w5Text$(1) = "Plotting to Printer."
               w5Text$(2) = "Please Wait."
               VGAplot w5, w5Text$(), w5Title$, w5Prompt$
               GOSUB PrintHeader
               GOSUB PrintPhi
               GOSUB SaveRawDat
               GOSUB PrintPlot
               GOSUB ToDisk
               windowspop
               ifin = 1
          END IF
          GOTO StartOver
   
     CASE 10
          GOSUB Filter
          GOSUB PhiCalc
          GOTO ScreenPlot

     CASE 11
          IF ifin = 0 THEN
               w1Text$(1) = "You have NOT saved data."
               w1Text$(2) = "Please select another option"
               GOSUB ErrWindow2
               code% = 27
               GOTO PullSelect
          END IF
          SYSTEM

     CASE ELSE
          GOTO keyin
     END SELECT

StartOver:

' ************************ CLEAN UP AND START NEXT ***************************
     ln1$ = "PLEASE ": ln2$ = "ENTER"
     sid$ = "PLEASE ENTER"
     SW = 0: CW = 0: SNDW = 0
     FOR i = 1 TO 5: GF(i) = 0: NEXT i: REM ZERO GF
     ' Initialize the display
     SCREEN 0, , 0, 0
     WIDTH 80
     CLS
     GOTO Begin
' ************************ END OF MAIN PROGRAM ******************************

' **************************** FILTER ***************************************
Filter:
     IFILT = 5
     FOR i = 11 TO Count% - IFILT
          x = 0
          FOR j = i TO i + (IFILT - 1)
          x = x + IV(j)
          NEXT j
          x = x / IFILT
          IV(i) = x
     NEXT i
     FOR i = Count% - IFILT TO Count%
          IV(i) = IV(Count% - IFILT)
     NEXT i
RETURN

PrintPlot:
     LPRINT "******************************** PLOT ****************************************"
'    PRINT "PLOTTING"
     LPRINT CHR$(27); "A"; CHR$(8): REM 8/72 SPACING FOR PLOT
     LPRINT CHR$(27); "K"; CHR$(225); CHR$(1); : REM 480 COLUMNS
     FOR i = 1 TO 480
          LPRINT CHR$(1);
     NEXT i: LPRINT
     FOR j = 200 TO 1 STEP -8: REM 25 LINES
'         PRINT J
          HILIM = j: LOLIM = j - 7: IPC = 1
          LPRINT CHR$(27); "K"; CHR$(225); CHR$(1); : REM 480 COLUMNS
          FOR i = 1 TO 480
          IF i = 1 OR i = 480 THEN LPRINT CHR$(255); : GOTO plnext
          IP = i * Iplot: IF IP > ICount THEN GOTO plclse
          IF (IP * TFAC) >= PHIT(IPC) THEN LPRINT CHR$(255); : IPC = IPC + 1: GOTO plnext
          IVAL = INT((IV(IP) + OFFSET) / RINC)
          IF IVAL < LOLIM THEN GOTO plclse
          IF IVAL > HILIM THEN GOTO plclse
          LPRINT CHR$(2 ^ (IVAL - LOLIM));
          GOTO plnext
plclse:
          LPRINT CHR$(0);
plnext:
          NEXT i
          LPRINT
     NEXT j
     LPRINT CHR$(27); "K"; CHR$(225); CHR$(1);
     FOR i = 1 TO 480: LPRINT CHR$(128); : NEXT i: LPRINT
     LPRINT CHR$(27); "2";
     LPRINT CHR$(12); CHR$(27); "@"; : REM TOF AND RESTORE PRINTER
RETURN

ToDisk:
     '************* SAVE TO DISK *************
     OPEN "R", #1, "RSA.NDX", 9    'changed to 9 for compatability w/CLTRQB93
     FIELD #1, 5 AS DISC$, 1 AS Stat$, 3 AS RSA$       ' AHE 8-93
     GET #1, 1: REM READ REC #1
     NFILES = VAL(DISC$)
     LSET DISC$ = STR$(NFILES + 1)
     PUT #1, 1
     LSET DISC$ = ln1$ + ln2$
     LSET Stat$ = "O"
     LSET RSA$ = "RSA"                                 ' AHE 8-93
     PUT #1, NFILES + 2
     CLOSE
     a$ = ln1$ + ln2$
     OPEN "R", #1, "RSA.DAT", 181
     FIELD #1, 6 AS QM$, 5 AS QL$, 3 AS QT$, 16 AS QS$, 16 AS QP$, 16 AS QC$, 16 AS QR$, 16 AS QO$, 8 AS QD$, 8 AS QH$
     FIELD #1, 110 AS DUMMY$, 7 AS QSW$, 7 AS QCW$, 7 AS QSN$, 5 AS QP$(0), 5 AS QP$(1), 5 AS QP$(2), 5 AS QP$(3), 5 AS QP$(4), 5 AS QP$(5), 5 AS QP$(6), 5 AS QP$(7), 5 AS QP$(8), 5 AS QP$(9)
     LSET QM$ = " ": LSET QL$ = a$: LSET QT$ = "RSA": LSET QS$ = sid$: LSET QP$ = PID$: LSET QC$ = CID$: LSET QR$ = SN$: LSET QO$ = OPN$
   
'    Y2K - y$ is complete year and all four digits are set into QD$
'    New format is MMDDYYYY
     LSET QD$ = RIGHT$(STR$(Mo), 2) + DA$ + RIGHT$(y$, 4): LSET QH$ = h$ + ":" + MI$ + ":" + S$
     LSET QSW$ = RIGHT$(STR$(SW), 7): LSET QCW$ = RIGHT$(STR$(CW), 7): LSET QSN$ = RIGHT$(STR$(SNDW), 7)
     FOR i = 0 TO 4: LSET QP$(i) = RIGHT$(STR$(GF(5 - i)), 5): NEXT i
     FOR i = 5 TO 9: LSET QP$(i) = RIGHT$(STR$(PHIR(i - 3)), 5): NEXT i
     PUT #1, NFILES + 1
     CLOSE
RETURN

readAtoD:
     IF Itest = 1 THEN
          '  quickbasic test routine
          IF TIMER < bt + 5 THEN RETURN
          VI = VI + 8.7539E-04: REM  increment starts at pi/2 , inc = pi/3600
          IGR = (SIN(VI) + 1) * 4000
     ELSE
        GOSUB ReadA2D
     END IF
     RETURN

clock:
     t$ = TIME$
     D$ = DATE$
     h$ = LEFT$(t$, 2)
     MI$ = MID$(t$, 4, 2)
     S$ = RIGHT$(t$, 2)
     Mo$ = LEFT$(D$, 2)
     Mo = VAL(Mo$)
     DA$ = MID$(D$, 4, 2)

'    Y2K  note that y$ is four digit at this point
     y$ = RIGHT$(D$, 4)
     RETURN


'*************** PRINT HEADER INFORMATION ***************
PrintHeader:
     LPRINT "Lab Number "; ln1$; ln2$
     LPRINT "Operator: "; OPN$; "   Requested by: "; SN$
     LPRINT "Cruise I.D.      "; CID$
     LPRINT "Project I.D.     "; PID$
     LPRINT "Sample I.D.      "; sid$
     LPRINT "Sample weight:   "; SW
     LPRINT "Coarse weight:   "; CW
     LPRINT "Sand weight:     "; SNDW
     IF IGF THEN
          LPRINT "Gravel Fraction Percentages:"
          FOR i = 1 TO 5
          LPRINT -1 * i; " Phi % = "; GF(i)
          NEXT i
     END IF
     LPRINT h$; ":"; MI$; ":"; S$; " "; Mo$(Mo); " "; DA$; ", "; y$
     LPRINT
     RETURN
PhiCalc:
' **************** COMPUTE PHI % **************
' ****** FFAC = POPPE TIME DELAY FACTOR. 1 UNIT = TFAC  SEC
' ****** TFAC = A/D CONVERT TIME BETWEEN SAMPLES [NOW .05]
' ****** PHIT(N) =  TIME COUNT WHEN AT .5 PHI CROSSOVER POINT
' ****** PHIV(N) =  PHI RAW VALUES AT CROSSOVER POINTS
' ****** PHIP(N) =  PHI PERCENTAGES AT .5 PHI POINTS
' ****** PHIR(N) =  PHI % ROUNDED TO .1% AT 1 PHI INTERVALS
    
     FOR i = 1 TO 12
          IP = INT(PHIT(i) / TFAC): REM PHI DATA POINT
          IF IP + FFAC > Count% THEN
          PHIV(i) = MIN
          ELSE PHIV(i) = IV(IP + FFAC)
          END IF
          ' Set any value less than initial value to the initial value
          IF PHIV(i) > PHIV(1) THEN PHIV(1) = PHIV(i)
     NEXT i
     FOR i = 1 TO 12
          FOR j = i TO 12
               IF PHIV(j) > PHIV(i) THEN PHIV(i) = PHIV(j)
          NEXT j
     NEXT i
     RANGE = PHIV(1) - MIN: REM MIN FROM SCALING ROUTINE
     FOR i = 1 TO 11
         PHIP(i + 1) = (PHIV(i) - PHIV(i + 1)) / (PHIV(1) + OFFSET) * 100
     NEXT i
     RETURN
    

PrintPhi:
     LPRINT : LPRINT "PHI          VALUE    PERCENTAGE": LPRINT
     FOR i = 1 TO 12
          IF i MOD 2 <> 1 THEN PHIR(i / 2) = (INT(((PHIP(i) + PHIP(i - 1)) * 10) + .5)) / 10
     NEXT i
     x = 0
     Imax% = 1: PhirMax = 0
     FOR i = 1 TO 6
          IF PHIR(i) > PhirMax THEN
               PhirMax = PHIR(i)
               Imax% = i
          END IF
          x = x + PHIR(i)
     NEXT i
     IF ABS(100 - x) < .001 THEN
          x = 100     'Added ABS to take care of sums >100 AHE 4-13-94
     ELSE
          IF (100 - x) > 0 THEN
               PhirMax = PHIR(6)             'save old value of 4 Phi
               PHIR(6) = PHIR(6) + (100 - x)
               Imax% = 6
               ' add the remainder to 4 Phi value
          END IF
          IF (100 - x) < 0 THEN PHIR(Imax%) = PHIR(Imax%) + (100 - x)
               ' add the (-) remainder to the biggest phi percentage
               ' ends up as a subtraction
     END IF
     'Round again
     PHIR(Imax%) = (INT((PHIR(Imax%) * 10) + .5)) / 10
     Phi = -2!
     FOR i = 1 TO 12
          Phi = Phi + .5
          LPRINT USING "+#.#"; Phi;
          LPRINT USING "############"; PHIV(i);
          LPRINT USING "########.###_%"; PHIP(i);
          IF i MOD 2 <> 1 THEN
               LPRINT USING "########.#_%"; PHIR(i / 2)
          ELSE LPRINT
          END IF
     NEXT i
     LPRINT : LPRINT "Total percentage = "; x; "   Remainder percentage = "; 100 - x
     PHIR(2) = PHIR(2) + PHIR(1): REM ADD -1 PHI TO 0 PHI
     LPRINT "New 0 Phi = "; PHIR(2); "%";
          IF x <> 100 THEN LPRINT " Changed"; Imax% - 2; " Phi from"; PhirMax; "% to "; PHIR(Imax%); " %";
          LPRINT
     RETURN
phiwindow:
     OPEN "PHIDUMMY" FOR OUTPUT AS #3
     PRINT #3, "PHI          VALUE    PERCENTAGE"
     PRINT #3,
     FOR i = 1 TO 12
          IF i MOD 2 <> 1 THEN PHIR(i / 2) = (INT(((PHIP(i) + PHIP(i - 1)) * 10) + .5)) / 10
     NEXT i


     x = 0
     Imax% = 1: PhirMax = 0
     FOR i = 1 TO 6
          IF PHIR(i) > PhirMax THEN
               PhirMax = PHIR(i)
               Imax% = i
          END IF
          x = x + PHIR(i)
     NEXT i
     IF ABS(100 - x) < .001 THEN
          x = 100     'Added ABS to take care of sums >100 AHE 4-13-94
     ELSE
          IF (100 - x) > 0 THEN
               Imax% = 6
               PhirMax = PHIR(6)
               PHIR(6) = PHIR(6) + (100 - x)
               ' add the remainder to 4 Phi value
          END IF
                   
          IF (100 - x) < 0 THEN PHIR(Imax%) = PHIR(Imax%) + (100 - x)
          ' add the (-) remainder to the biggest phi percentage
          ' ends up as a subtraction
     END IF
    
     PHIR(Imax%) = (INT((PHIR(Imax%) * 10) + .5)) / 10

     Phi = -2!
     FOR i = 1 TO 12
          Phi = Phi + .5
          PRINT #3, USING "+#.#"; Phi;
          PRINT #3, USING "############"; PHIV(i);
          PRINT #3, USING "########.###_%"; PHIP(i);
          IF i MOD 2 <> 1 THEN
          PRINT #3, USING "########.#_%"; PHIR(i / 2)
          ELSE PRINT #3,
          END IF
     NEXT i
     PRINT #3, : PRINT #3, "Total percentage = "; x; "   Remainder percentage = "; 100 - x
     PHIR(2) = PHIR(2) + PHIR(1): REM ADD -1 PHI TO 0 PHI
     PRINT #3, "New 0 Phi = "; PHIR(2); "%";
     IF x <> 100 THEN PRINT #3, " Chngd"; Imax% - 2; " Phi from"; PhirMax; "% to"; PHIR(Imax%); "%";
     CLOSE #3
     OPEN "PHIDUMMY" FOR INPUT AS #3
     FOR i = 1 TO 17
          INPUT #3, w2Text$(i)
     NEXT i
     CLOSE #3
     RETURN

timeout:           '1 second timeout
     t0 = TIMER
     DO
     LOOP UNTIL TIMER - t0 > 1
     RETURN
ErrWindow1:
     Windows w1, w1Text$(), w1Title$, w1Prompt$
     BEEP
     GOSUB timeout
     windowspop
     w1Text$(1) = "Incorrect information entered."
     w1Text$(2) = "Please re-enter."
     RETURN

ErrWindow2:
     VGAplot w1, w1Text$(), w1Title$, w1Prompt$
     BEEP
     GOSUB timeout
     windowspop
     w1Text$(1) = "Incorrect information entered."
     w1Text$(2) = "Please re-enter."
     RETURN
SaveRawDat:
     'OPEN "RawIndex.dat" FOR OUTPUT AS #4
     'PRINT #4, 1
     'CLOSE #4
     OPEN "RawIndex.dat" FOR INPUT AS #4        'Will crash if file missing
     INPUT #4, RawPtr%
     CLOSE #4
     '    PRINT RawPtr%
     RawPtr% = RawPtr% + 1
     IF RawPtr% > 3 THEN RawPtr% = 1
     OPEN "RawDat" + RIGHT$(STR$(RawPtr%), 1) + ".dat" FOR OUTPUT AS #4
     PRINT #4, Count%
     PRINT #4, ln1$
     PRINT #4, ln2$
     PRINT #4, sid$
     FOR i = 1 TO Count%
          PRINT #4, IV(i)
     NEXT i
     CLOSE #4
     OPEN "RawIndex.dat" FOR OUTPUT AS #4
     PRINT #4, RawPtr%
     CLOSE #4
     RETURN

GetRawDat:
     OPnum% = OPnum% + 1   'must init this to 0 before 1st call
     IF OPnum > 2 THEN OPnum = 0
     OPEN "RawIndex.dat" FOR INPUT AS #4
     INPUT #4, RawPtr%
     CLOSE #4
     '     RawPtr% is last saved file #
     RawPtr% = RawPtr% - OPnum%     'subtract call number
     IF RawPtr% < 1 THEN RawPtr% = RawPtr% + 3
     OPEN "RawDat" + RIGHT$(STR$(RawPtr%), 1) + ".dat" FOR INPUT AS #4
     INPUT #4, OPcount
     INPUT #4, OPln1$
     INPUT #4, OPln2$
     INPUT #4, OPsid$
     FOR i = 1 TO OPcount
          INPUT #4, IVOP(i)
     NEXT i
     CLOSE #4
     RETURN

InitA2D:
    ' Initialize ADLIB with the configuration file ADAC.DAT.
    HWCONFIG$ = "ADAC.DAT" + CHR$(0)
    Stat% = AINIT(VARSEG(HWCONFIG$), SADD(HWCONFIG$))
    IF (Stat% <> 0) THEN
        PRINT "Error during INIT, status ="; Stat%
        END
        SYSTEM
    END IF
    ' Initialize the on board counter/timer
    Fosc% = &HC350              ' 50,000 time units per sample at 1 MHz = 20 per sec
                                ' C350h=50000d unsigned integer > 32767
    Mode = 0                    ' Constant Frequency
    IChan = 0                   ' Just first chan (0)
    ILun = 1                    ' Logical Unit # 1
    Stat% = COT(VARSEG(Fosc%), VARPTR(Fosc%), Mode, IChan, ILun)
    IF (Stat% <> 0) THEN
        PRINT "Error during COT, status ="; Stat%
    END IF
    RETURN

ReadA2D:
    Stat% = AIN(VARSEG(Buff%(0)), VARPTR(Buff%(0)), ICount, IChan, ITrig, IGain, ILun)
    IF Stat% <> 0 THEN
        PRINT "Error during AIN, status ="; Stat%
    END IF
    IGR = Buff%(0)
    RETURN

ScanA2D:
    Stat% = AINI(VARSEG(IV(IVOffset)), VARPTR(IV(IVOffset)), ICount, IChan, ITrig, IGain, ILun, Iflag)
    IF Stat% <> 0 THEN
        PRINT "Error during AINI, status ="; Stat%
    END IF
    RETURN

StopA2D:
    IF Itest THEN RETURN            ' Avoid crash - no A/D card
    Stat% = IOKIL(ILun)             ' Terminate I/O req.
    IF Stat% <> 0 THEN
        PRINT "Error during IOKIL, status ="; Stat%
        END
    END IF
  
    Stat% = ARESTORE
    IF (Stat% <> 0) THEN
        PRINT "Error during ARESTORE, status ="; Stat%
        PRINT "System must be rebooted "
        END
    END IF
    RETURN

' ******************************** END OF PROGRAM *******************************

DEFSNG I-M
  ' ************************************************
  ' **  Name:          VGAplot                    **
  ' **  Type:          Subprogram                 **
  ' **  Module:        VGAplot                    **
  ' **  Language:      Microsoft QuickBASIC 4.00  **
  ' ************************************************
  '
  ' Sets up a rectangular window and plots to it using variables passed
  ' from the calling program. Position variables are the same as those
  ' used for the Windows subprogram.
  ' This module works similar to WINDOWS but is used to create plot windows
  ' for the VGA operating in Mode 10. Only one additional window may be pulled
  ' down over the first for error messages or menus. Intensities are set by
  ' default within this module. After a call to this routine the page may be
  ' set to 0 by PCOPY 1,0 and Screen , ,0,0 in calling program.
  '
  ' EXAMPLE OF USE:  VGAplot w1, wText$(), wTitle$, wPrompt$
  ' PARAMETERS:      w1            Structure of type WindowsType
  '                  wTest$()      Array of strings to be displayed
  '                  wTitle$       Title string
  '                  wPrompt$      Prompt string
  ' VARIABLES:       mode%         Current video mode
  '                  columns%      Current number of character columns
  '                  page%         Current video page
  '                  cursorRow%    Saved cursor row position
  '                  cursorCol%    Saved cursor column position
  '                  newpage%      Next video page
  '                  lbText%       Lower boundary of array of text lines
  '                  ubText%       Upper boundary of array of text lines
  '                  i%            Looping index
  '                  maxlen%       Length of longest string to display
  '                  length%       Length of each array string
  '                  row2%         Row number at bottom right corner of window
  '                  col2%         Column number at bottom right corner of
  '                                window
  '                  ul%           Upper left corner border character code
  '                  ur%           Upper right corner border character code
  '                  ll%           Lower left corner border character code
  '                  lr%           Lower right corner border character code
  '                  vl%           Vertical border character code
  '                  hl%           Horizontal border character code
  '                  r%            Index to each line of text
  '                  ptr%          Highlighted line pointer
  '                  lastPtr%      Last highlighted line
  '                  horizontal%   Horizontal mouse mickies
  '                  vertical%     Vertical mouse mickies
  '                  mickies       Accumulated vertical mickies
  '                  choice$       Set of unique characters for each menu line
  '                  tmp$          Work string
  '                  kee%          Key code returned by InKeyCode% function
  '                  leftButton%   Mouse left button state
  '                  rightButton%  Mouse right button state
  '                  xMouse%       Mouse X position
  '                  yMouse%       Mouse Y position
  ' MODULE LEVEL
  '   DECLARATIONS:  SUB Windows (w AS WindowsType, wText$(), wTitle$,
  '                               wPrompt$) STATIC
  '
    SUB VGAplot (W AS WindowsType, wText$(), wTitle$, wPrompt$) STATIC
      ' Key code numbers
        CONST DOWNARROW = 20480
        CONST ENTER = 13
        CONST ESCAPE = 27
        CONST UPARROW = 18432
   
      ' Determine current video page
        VideoState Mode%, columns%, page%
   
      ' Record current cursor location
        cursorRow% = CSRLIN
        cursorCol% = POS(0)
   
      ' Window will be on the next page, if available
        newpage% = page% + 1
        IF newpage% > 1 THEN
            SCREEN , , 0, 0
            PRINT "Error: Windows - not enough video pages"
            SYSTEM
        END IF
   
      ' Copy current page to new page
        PCOPY page%, newpage%
   
      ' Show the current page while building window on new page
        SCREEN , , newpage%, page%
   
      ' Determine array bounds
        lbText% = LBOUND(wText$)
        ubText% = UBOUND(wText$)
   
      ' Check the text array bounds, lower always 1, upper > 0
        IF lbText% <> 1 OR ubText% < 1 THEN
            SCREEN , , 0, 0
            PRINT "Error: Windows - text array dimensioned incorrectly"
            SYSTEM
        END IF
           
      ' Determine longest string in the text array
        maxlen% = 0
        FOR i% = lbText% TO ubText%
            length% = LEN(wText$(i%))
            IF length% > maxlen% THEN
                maxlen% = length%
            END IF
        NEXT i%
    SELECT CASE W.action
        CASE 3
      ' Expand to fill to bottom and right screen
        row2% = 24
        col2% = 80 - W.col
        maxlen% = col2% - 3
      ' Open a plot window within the screen.
        VIEW (1 + W.col * 8, 1 + W.row * 14)-(631, 321), 3, 1

        CASE ELSE
      ' Determine the bottom right corner of window
        row2% = W.row + ubText% + 1
        col2% = W.col + maxlen% + 3
    END SELECT

      ' Check that window is on screen
        IF W.row < 1 OR W.col < 1 OR (row2% > 25 AND W.action = 3) OR (row2% > 25 AND W.action <> 3) OR col2% > columns% THEN
            SCREEN , , 0, 0
            PRINT "Error: Windows - part of window is off screen"
            SYSTEM
        END IF
  
      ' Set the INTENSITY parameters
        W.fgdEdge = 1: REM NORMAL
        W.bgdEdge = 0: REM OFF
        W.fgdBody = 1
        W.bgdBody = 0
        W.fgdHighlight = 3: REM BRIGHT
        W.bgdHighlight = 0
        W.fgdTitle = 3
        W.bgdTitle = 0
        W.fgdPrompt = 3
        W.bgdPrompt = 0

      ' Set the edge characters
        SELECT CASE W.edgeLine
        CASE 0
            ul% = 32
            ur% = 32
            ll% = 32
            lr% = 32
            vl% = 32
            hl% = 32
        CASE 1
            ul% = 218
            ur% = 191
            ll% = 192
            lr% = 217
            vl% = 179
            hl% = 196
        CASE 2
            ul% = 201
            ur% = 187
            ll% = 200
            lr% = 188
            vl% = 186
            hl% = 205
        CASE ELSE
            SCREEN , , 0, 0
            PRINT "Error: Windows - Edge line type incorrect"
            SYSTEM
        END SELECT
   
      ' Draw top edge of the box
        LOCATE W.row, W.col, 0
        COLOR W.fgdEdge, W.bgdEdge
        PRINT CHR$(ul%); STRING$(maxlen% + 2, hl%); CHR$(ur%);
   
      ' Draw the body of the window
        FOR R% = W.row + 1 TO row2% - 1
            LOCATE R%, W.col, 0
            COLOR W.fgdEdge, W.bgdEdge
            PRINT CHR$(vl%);
            COLOR W.fgdBody, W.bgdBody
            IF R% - W.row <= ubText% THEN
                tmp$ = LEFT$(wText$(R% - W.row) + SPACE$(maxlen%), maxlen%)
                ELSE tmp$ = SPACE$(maxlen%)
            END IF
            PRINT " "; tmp$; " ";
           
            COLOR W.fgdEdge, W.bgdEdge
            PRINT CHR$(vl%);
        NEXT R%
   
      ' Draw bottom edge of the box
        LOCATE row2%, W.col, 0
        COLOR W.fgdEdge, W.bgdEdge
        PRINT CHR$(ll%); STRING$(maxlen% + 2, hl%); CHR$(lr%);
   
      ' Center and print top title if present
        IF wTitle$ <> "" THEN
            LOCATE W.row, (W.col + col2% - LEN(wTitle$) + 1) \ 2, 0
            COLOR W.fgdTitle, W.bgdTitle
            PRINT wTitle$;
        END IF
   
      ' Center and print prompt if present
        IF wPrompt$ <> "" THEN
            LOCATE row2%, (W.col + col2% - LEN(wPrompt$) + 1) \ 2, 0
            COLOR W.fgdPrompt, W.bgdPrompt
            PRINT wPrompt$;
        END IF
   
      ' Now make the new page visible and active
        SCREEN , , newpage%, newpage%
   
      ' Take next action based on action code
        SELECT CASE W.action
        CASE 1
       
          ' Get a key code number and return it
            DO
                W.ReturnCode = InKeyCode%
            LOOP UNTIL W.ReturnCode
       
        CASE 2
       
          ' Set choice pointer to last selection if known
            IF W.ReturnCode > 0 AND W.ReturnCode < ubText% THEN
                ptr% = W.ReturnCode
            ELSE
                ptr% = 1
            END IF
       
          ' Start with last pointer different, to update highlighting
            IF ptr% > 1 THEN
                lastPtr% = 1
            ELSE
                lastPtr% = 2
            END IF
       
          ' Clear any mouse mickey counts
            MouseMickey horizontal%, vertical%
            mickies% = 0
       
          ' Create unique key selection string
            choice$ = ""
            FOR i% = 1 TO ubText%
                tmp$ = UCASE$(LTRIM$(wText$(i%)))
                DO
                    IF tmp$ <> "" THEN
                        t$ = LEFT$(tmp$, 1)
                        tmp$ = MID$(tmp$, 2)
                        IF INSTR(choice$, t$) = 0 THEN
                            choice$ = choice$ + t$
                        END IF
                    ELSE
                        SCREEN 0, , 0
                        PRINT "Error: Windows - No unique character"
                        SYSTEM
                    END IF
                LOOP UNTIL LEN(choice$) = i%
            NEXT i%
       
          ' Main loop, monitor mouse and keyboard
            DO
           
              ' Add the mouse mickies
                MouseMickey horizontal%, vertical%
                mickies% = mickies% + vertical%
           
              ' Check for enough mickies
                IF mickies% < -17 THEN
                    mickies% = 0
                    IF ptr% > 1 THEN
                        ptr% = ptr% - 1
                    END IF
                ELSEIF mickies% > 17 THEN
                    mickies% = 0
                    IF ptr% < ubText% THEN
                        ptr% = ptr% + 1
                    END IF
                END IF
           
              ' Check keyboard
                kee% = InKeyCode%
                IF kee% >= ASC("a") AND kee% <= ASC("z") THEN
                    kee% = ASC(UCASE$(CHR$(kee%)))
                END IF
                SELECT CASE kee%
                CASE UPARROW
                    IF ptr% > 1 THEN
                        ptr% = ptr% - 1
                    END IF
                CASE DOWNARROW
                    IF ptr% < ubText% THEN
                        ptr% = ptr% + 1
                    END IF
                CASE ENTER
                    W.ReturnCode = ptr%
                CASE ESCAPE
                    W.ReturnCode = -1
                CASE ELSE
                    IF (kee% >= ASC("0") AND kee% <= ASC("9")) OR (kee% >= ASC("A") AND kee% <= ASC("Z")) THEN
                         W.ReturnCode = INSTR(choice$, CHR$(kee%))
                         IF W.ReturnCode THEN
                         ptr% = W.ReturnCode
                         END IF
                    ELSE W.ReturnCode = 0
                    END IF
                END SELECT
           
              ' Check the left mouse button
                MouseNow leftButton%, rightButton%, xMouse%, yMouse%
                IF leftButton% THEN
                    W.ReturnCode = ptr%
                END IF
           
              ' Update the highlight if line has changed
                IF ptr% <> lastPtr% THEN
                    LOCATE lastPtr% + W.row, W.col + 2, 0
                    COLOR W.fgdBody, W.bgdBody
                    tmp$ = LEFT$(wText$(lastPtr%) + SPACE$(maxlen%), maxlen%)
                    PRINT tmp$;
                    LOCATE ptr% + W.row, W.col + 2, 0
                    COLOR W.fgdHighlight, W.bgdHighlight
                    tmp$ = LEFT$(wText$(ptr%) + SPACE$(maxlen%), maxlen%)
                    PRINT tmp$;
                    lastPtr% = ptr%
                END IF
           
            LOOP WHILE W.ReturnCode = 0

        CASE ELSE
            W.ReturnCode = 0
        END SELECT
   
      ' Reset the cursor position
        LOCATE cursorRow%, cursorCol%
   
    END SUB

  ' ************************************************
  ' **  Name:          Windows                    **
  ' **  Type:          Subprogram                 **
  ' **  Module:        WINDOWS.BAS                **
  ' **  Language:      Microsoft QuickBASIC 4.00  **
  ' ************************************************
  '
  ' Displays a rectangular window for information display
  ' or menu selection.
  '
  ' EXAMPLE OF USE:  Windows w1, wText$(), wTitle$, wPrompt$
  ' PARAMETERS:      w1            Structure of type WindowsType
  '                  wTest$()      Array of strings to be displayed
  '                  wTitle$       Title string
  '                  wPrompt$      Prompt string
  ' VARIABLES:       mode%         Current video mode
  '                  columns%      Current number of character columns
  '                  page%         Current video page
  '                  cursorRow%    Saved cursor row position
  '                  cursorCol%    Saved cursor column position
  '                  newpage%      Next video page
  '                  lbText%       Lower boundary of array of text lines
  '                  ubText%       Upper boundary of array of text lines
  '                  i%            Looping index
  '                  maxlen%       Length of longest string to display
  '                  length%       Length of each array string
  '                  row2%         Row number at bottom right corner of window
  '                  col2%         Column number at bottom right corner of
  '                                window
  '                  ul%           Upper left corner border character code
  '                  ur%           Upper right corner border character code
  '                  ll%           Lower left corner border character code
  '                  lr%           Lower right corner border character code
  '                  vl%           Vertical border character code
  '                  hl%           Horizontal border character code
  '                  r%            Index to each line of text
  '                  ptr%          Highlighted line pointer
  '                  lastPtr%      Last highlighted line
  '                  horizontal%   Horizontal mouse mickies
  '                  vertical%     Vertical mouse mickies
  '                  mickies       Accumulated vertical mickies
  '                  choice$       Set of unique characters for each menu line
  '                  tmp$          Work string
  '                  kee%          Key code returned by InKeyCode% function
  '                  leftButton%   Mouse left button state
  '                  rightButton%  Mouse right button state
  '                  xMouse%       Mouse X position
  '                  yMouse%       Mouse Y position
  ' MODULE LEVEL
  '   DECLARATIONS:  SUB Windows (w AS WindowsType, wText$(), wTitle$,
  '                               wPrompt$) STATIC
  '
    SUB Windows (W AS WindowsType, wText$(), wTitle$, wPrompt$) STATIC
      ' Key code numbers
        CONST DOWNARROW = 20480
        CONST ENTER = 13
        CONST ESCAPE = 27
        CONST UPARROW = 18432
     
      ' Determine current video page
        VideoState Mode%, columns%, page%
     
      ' Record current cursor location
        cursorRow% = CSRLIN
        cursorCol% = POS(0)
     
      ' Window will be on the next page, if available
        newpage% = page% + 1
        IF newpage% > 7 THEN
            SCREEN , , 0, 0
            PRINT "Error: Windows - not enough video pages"
            SYSTEM
        END IF
     
      ' Copy current page to new page
        PCOPY page%, newpage%
     
      ' Show the current page while building window on new page
        SCREEN , , newpage%, page%
     
      ' Determine array bounds
        lbText% = LBOUND(wText$)
        ubText% = UBOUND(wText$)
     
      ' Check the text array bounds, lower always 1, upper > 0
        IF lbText% <> 1 OR ubText% < 1 THEN
            SCREEN , , 0, 0
            PRINT "Error: Windows - text array dimensioned incorrectly"
            SYSTEM
        END IF
     
      ' Determine longest string in the text array
        maxlen% = 0
        FOR i% = lbText% TO ubText%
            length% = LEN(wText$(i%))
            IF length% > maxlen% THEN
                maxlen% = length%
            END IF
        NEXT i%
     
      ' Determine the bottom right corner of window
        row2% = W.row + ubText% + 1
        col2% = W.col + maxlen% + 3
     
      ' Check that window is on screen
        IF W.row < 1 OR W.col < 1 OR row2% > 25 OR col2% > columns% THEN
            SCREEN , , 0, 0
            PRINT "Error: Windows - part of window is off screen"
            SYSTEM
        END IF
     
      ' Set the edge characters
        SELECT CASE W.edgeLine
        CASE 0
            ul% = 32
            ur% = 32
            ll% = 32
            lr% = 32
            vl% = 32
            hl% = 32
        CASE 1
            ul% = 218
            ur% = 191
            ll% = 192
            lr% = 217
            vl% = 179
            hl% = 196
        CASE 2
            ul% = 201
            ur% = 187
            ll% = 200
            lr% = 188
            vl% = 186
            hl% = 205
        CASE ELSE
            SCREEN , , 0, 0
            PRINT "Error: Windows - Edge line type incorrect"
            SYSTEM
        END SELECT
     
      ' Draw top edge of the box
        LOCATE W.row, W.col, 0
        COLOR W.fgdEdge, W.bgdEdge
        PRINT CHR$(ul%); STRING$(maxlen% + 2, hl%); CHR$(ur%);
     
      ' Draw the body of the window
        FOR R% = W.row + 1 TO row2% - 1
            LOCATE R%, W.col, 0
            COLOR W.fgdEdge, W.bgdEdge
            PRINT CHR$(vl%);
            COLOR W.fgdBody, W.bgdBody
            tmp$ = LEFT$(wText$(R% - W.row) + SPACE$(maxlen%), maxlen%)
            PRINT " "; tmp$; " ";
            COLOR W.fgdEdge, W.bgdEdge
            PRINT CHR$(vl%);
        NEXT R%
     
      ' Draw bottom edge of the box
        LOCATE row2%, W.col, 0
        COLOR W.fgdEdge, W.bgdEdge
        PRINT CHR$(ll%); STRING$(maxlen% + 2, hl%); CHR$(lr%);
     
      ' Center and print top title if present
        IF wTitle$ <> "" THEN
            LOCATE W.row, (W.col + col2% - LEN(wTitle$) + 1) \ 2, 0
            COLOR W.fgdTitle, W.bgdTitle
            PRINT wTitle$;
        END IF
     
      ' Center and print prompt if present
        IF wPrompt$ <> "" THEN
            LOCATE row2%, (W.col + col2% - LEN(wPrompt$) + 1) \ 2, 0
            COLOR W.fgdPrompt, W.bgdPrompt
            PRINT wPrompt$;
        END IF
     
      ' Now make the new page visible and active
        SCREEN , , newpage%, newpage%
     
      ' Take next action based on action code
        SELECT CASE W.action
        CASE 1
         
          ' Get a key code number and return it
            DO
                W.ReturnCode = InKeyCode%
            LOOP UNTIL W.ReturnCode
         
        CASE 2
         
          ' Set choice pointer to last selection if known
            IF W.ReturnCode > 0 AND W.ReturnCode < ubText% THEN
                ptr% = W.ReturnCode
            ELSE
                ptr% = 1
            END IF
         
          ' Start with last pointer different, to update highlighting
            IF ptr% > 1 THEN
                lastPtr% = 1
            ELSE
                lastPtr% = 2
            END IF
         
          ' Clear any mouse mickey counts
            MouseMickey horizontal%, vertical%
            mickies% = 0
         
          ' Create unique key selection string
            choice$ = ""
            FOR i% = 1 TO ubText%
                tmp$ = UCASE$(LTRIM$(wText$(i%)))
                DO
                    IF tmp$ <> "" THEN
                        t$ = LEFT$(tmp$, 1)
                        tmp$ = MID$(tmp$, 2)
                        IF INSTR(choice$, t$) = 0 THEN
                            choice$ = choice$ + t$
                        END IF
                    ELSE
                        SCREEN 0, , 0
                        PRINT "Error: Windows - No unique character"
                        SYSTEM
                    END IF
                LOOP UNTIL LEN(choice$) = i%
            NEXT i%
         
          ' Main loop, monitor mouse and keyboard
            DO
             
              ' Add the mouse mickies
                MouseMickey horizontal%, vertical%
                mickies% = mickies% + vertical%
             
              ' Check for enough mickies
                IF mickies% < -17 THEN
                    mickies% = 0
                    IF ptr% > 1 THEN
                        ptr% = ptr% - 1
                    END IF
                ELSEIF mickies% > 17 THEN
                    mickies% = 0
                    IF ptr% < ubText% THEN
                        ptr% = ptr% + 1
                    END IF
                END IF
             
              ' Check keyboard
                kee% = InKeyCode%
                IF kee% >= ASC("a") AND kee% <= ASC("z") THEN
                    kee% = ASC(UCASE$(CHR$(kee%)))
                END IF
                SELECT CASE kee%
                CASE UPARROW
                    IF ptr% > 1 THEN
                        ptr% = ptr% - 1
                    END IF
                CASE DOWNARROW
                    IF ptr% < ubText% THEN
                        ptr% = ptr% + 1
                    END IF
                CASE ENTER
                    W.ReturnCode = ptr%
                CASE ESCAPE
                    W.ReturnCode = -1
                CASE ELSE
                    W.ReturnCode = INSTR(choice$, CHR$(kee%))
                    IF W.ReturnCode THEN
                        ptr% = W.ReturnCode
                    END IF
                END SELECT
             
              ' Check the left mouse button
                MouseNow leftButton%, rightButton%, xMouse%, yMouse%
                IF leftButton% THEN
                    W.ReturnCode = ptr%
                END IF
             
              ' Update the highlight if line has changed
                IF ptr% <> lastPtr% THEN
                    LOCATE lastPtr% + W.row, W.col + 2, 0
                    COLOR W.fgdBody, W.bgdBody
                    tmp$ = LEFT$(wText$(lastPtr%) + SPACE$(maxlen%), maxlen%)
                    PRINT tmp$;
                    LOCATE ptr% + W.row, W.col + 2, 0
                    COLOR W.fgdHighlight, W.bgdHighlight
                    tmp$ = LEFT$(wText$(ptr%) + SPACE$(maxlen%), maxlen%)
                    PRINT tmp$;
                    lastPtr% = ptr%
                END IF
             
            LOOP WHILE W.ReturnCode = 0
         
        CASE ELSE
            W.ReturnCode = 0
        END SELECT
     
      ' Reset the cursor position
        LOCATE cursorRow%, cursorCol%
     
    END SUB

  ' ************************************************
  ' **  Name:          WindowsPop                 **
  ' **  Type:          Subprogram                 **
  ' **  Module:        WINDOWS.BAS                **
  ' **  Language:      Microsoft QuickBASIC 4.00  **
  ' ************************************************
  '
  ' Removes last displayed window.
  '
  ' EXAMPLE OF USE:  WindowsPop
  ' PARAMETERS:      (none)
  ' VARIABLES:       mode%      Current video mode
  '                  columns%   Current number of display columns
  '                  page%      Current display page
  ' MODULE LEVEL
  '   DECLARATIONS:  DECLARE SUB WindowsPop ()
  '
    SUB windowspop STATIC
        VideoState Mode%, columns%, page%
        IF page% THEN
            SCREEN , , page% - 1, page% - 1
        END IF
    END SUB

